@@ -230,21 +230,22 @@
 
 ;; for #:session
 (define (session-maker mode rule keys)
-  (define* (%spawn rc #:key (keep? #f) (idname "sid") (proc session-spawn))
+  (define* (%spawn rc #:key (idname "sid") (proc session-spawn))
     (call-with-values
         (lambda () (proc rc))
       (lambda (sid session)
-        (when keep?
-          (let ((cookie (new-cookie #:npv `((,idname ,sid)))))
-            (and cookie (rc-set-cookie! rc (list cookie))))))))
+        (let ((cookie (new-cookie #:npv `((,idname . ,sid)))))
+          (and cookie (rc-set-cookie! rc (list cookie)))
+          sid))))
   (define* (spawn-handler rc #:key (keep? #f))
     (match mode
-      ((or #t 'spawn) (%spawn rc #:keep? keep?))
-      (`(spawn ,sid) (%spawn rc #:idname sid #:keep? keep?))
-      (`(spawn ,sid ,proc) (%spawn rc #:idname sid #:proc proc #:keep? keep?))
+      ((or #t 'spawn) (%spawn rc))
+      (`(spawn ,sid) (%spawn rc #:idname sid))
+      (`(spawn ,sid ,proc) (%spawn rc #:idname sid #:proc proc))
       (else (throw 'artanis-err 500 "session-maker: Invalid config mode" mode))))
   (define (check-it rc idname)
-    (get-session (cookie-ref (rc-cookie rc) idname)))
+    (and=> (session-restore (cookie-ref (rc-cookie rc) idname))
+           (lambda (s) (session-from-correct-client? s rc))))
   (lambda (rc cmd)
     (match cmd
       ('check (check-it rc "sid"))
@@ -253,10 +254,7 @@
        (or (check-it rc "sid") (spawn-handler rc)))
       (`(check-and-spawn ,sid)
        (or (check-it rc sid) (spawn-handler rc)))
-      (`(check-and-spawn-and-keep ,sid)
-       (or (check-it rc sid) (spawn-handler rc #:keep? #t)))
       ('spawn (spawn-handler rc))
-      ('spawn-and-keep (spawn-handler rc #:keep? #t))
       (else (throw 'artanis-err 500 "session-maker: Invalid call cmd" cmd)))))
 
 (define (from-post-maker mode rule keys)
@@ -276,14 +274,19 @@
                  size-list filename-list))))
   (define (default-no-file-ret) "<p>No uploaded files!</p>")
   (define* (store-the-bv rc #:key (uid 33) (gid 33) (path (get-conf '(upload path)))
-                         (mod #o664) (path-mode #o775) (sync #f)
+                         (mode #o664) (path-mode #o775) (sync #f) (simple-ret? #t)
+                         (need-mfd? #f)
                          ;; NOTE: One may use these two things for returning
                          ;;       customized result, like JSON or XML.
                          (success-ret default-success-ret)
                          (no-file-ret "No uploaded files!"))
-    (match (store-uploaded-files rc #:path path #:sync sync #:simple-ret? #f
-                                 #:uid uid #:gid gid #:path-mode path-mode)
+    (match (store-uploaded-files rc #:path path #:sync sync #:simple-ret? simple-ret?
+                                 #:mode mode #:uid uid #:gid gid #:path-mode path-mode
+                                 #:need-mfd? need-mfd?)
+      ('success 'success)
+      ((? is-mfds? mfds) mfds)
       (`(success ,slist ,flist) (success-ret slist flist))
+      (`(success ,mfds ,slist ,flist) (success-ret mfds slist flist))
       ('none (no-file-ret))
       (else
        (throw 'artanis-err 500
@@ -295,9 +298,7 @@
       ('qstr-safe (post->qstr-table rc 'safe))
       ((or 'bv 'bytevector) (rc-body rc))
       ;; upload operation, indeed
-      (`(store ,path) (store-the-bv rc #:path path))
-      (`(store ,path ,mod)
-       (store-the-bv rc #:path path #:mod mod))
+      (('store rest ...) (apply store-the-bv rc rest))
       (else (throw 'artanis-err 500 "post-handler: Invalid mode!" mode))))
   (lambda (rc . cmd)
     (match cmd
