@@ -20,14 +20,18 @@
 (define-module (artanis utils)
   #:use-module (artanis crypto md5)
   #:use-module (artanis crypto sha-1)
+  #:use-module (artanis crypto base64)
   #:use-module (artanis tpl sxml)
   #:use-module (artanis config)
   #:use-module (artanis irregex)
   #:use-module (artanis env)
   #:use-module (artanis mime)
   #:use-module (system foreign)
+  #:use-module (ice-9 rdelim)
   #:use-module (ice-9 regex)
   #:use-module (ice-9 match)
+  #:use-module (ice-9 format)
+  #:use-module (ice-9 ftw)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-19)
   #:use-module (ice-9 local-eval)
@@ -35,6 +39,10 @@
   #:use-module (ice-9 q)
   #:use-module (web http)
   #:use-module (web request)
+  #:use-module ((rnrs)
+                #:select (get-bytevector-all utf8->string put-bytevector
+                          bytevector-u8-ref string->utf8 bytevector-length
+                          make-bytevector bytevector-s32-native-ref))
   #:export (regexp-split hash-keys cat bv-cat get-global-time
             get-local-time string->md5 unsafe-random string-substitute
             get-file-ext get-global-date get-local-date uri-decode
@@ -47,13 +55,23 @@
             new-stack new-queue stack-slots queue-slots stack-pop! stack-push!
             stack-top stack-empty? queue-out! queue-in! queue-head queue-tail
             queue-empty? list->stack list->queue stack-remove! queue-remove!
+            queue->list stack->list queue-length stack-length
             plist->alist make-db-string-template non-list?
             keyword->string range oah->handler oah->opts string->keyword
             alist->klist alist->kblist is-hash-table-empty?
-            symbol-downcase symbol-upcase normalize-column
+            symbol-downcase symbol-upcase normalize-column run-before-run!
             sxml->xml-string run-after-request! run-before-response!
             make-pipeline HTML-entities-replace eliminate-evil-HTML-entities
-            generate-kv-from-post-qstr)
+            generate-kv-from-post-qstr handle-proper-owner
+            generate-data-url verify-ENTRY
+            draw-expander remove-ext scan-app-components cache-this-route!
+            dump-route-from-cache generate-modify-time delete-directory
+            handle-existing-file check-drawing-method
+            subbv->string subbv=? bv-read-line bv-read-delimited put-bv
+            bv-u8-index bv-u8-index-right build-bv-lookup-table filesize
+            plist-remove gen-migrate-module-name try-to-load-migrate-cache
+            flush-to-migration-cache gen-local-conf-file with-dbd errno
+            c/struct-sizeof)
   #:re-export (the-environment))
 
 ;; There's a famous rumor that 'urandom' is safer, so we pick it.
@@ -65,12 +83,10 @@
         (if uppercase
             (string-upcase str)
             str)))))
-           
+
 (define uri-decode (@ (web uri) uri-decode))
 (define parse-date (@@ (web http) parse-date))
 (define write-date (@@ (web http) write-date))
-(define bytevector? (@ (rnrs) bytevector?))
-(define utf8->string (@ (rnrs) utf8->string))
 
 (define-syntax-rule (local-eval-string str e)
   (local-eval 
@@ -85,10 +101,10 @@
     ht))
 
 (eval-when (eval load compile)
- (define (export-all-from-module! module-name)
-   (let ((mod (resolve-module module-name)))
-         (module-for-each (lambda (s m) 
-                            (module-add! (current-module) s m)) mod))))
+           (define (export-all-from-module! module-name)
+             (let ((mod (resolve-module module-name)))
+               (module-for-each (lambda (s m) 
+                                  (module-add! (current-module) s m)) mod))))
 
 (define (time-expired? expires)
   (if expires
@@ -210,10 +226,17 @@
 (define-syntax-rule (static-filename path)
   (substring/shared path 1))
 
+(define-syntax-rule (request-ip req)
+  ;; TODO: support AF_INET6 in the future
+  (if (port-filename (request-port req))
+      ;; Valid socket port
+      (inet-ntop AF_INET (sockaddr:addr (getpeername (request-port req))))
+      "localtest")) ; fake hostname for testing
+
 (define-syntax-rule (remote-info req)
   (if (get-conf '(server nginx))
       (assoc-ref (request-headers req) 'x-real-ip)
-      (request-host req)))
+      (request-ip req)))
 
 (define *methods-list* '(HEAD GET POST PUT PATCH DELETE))
 (define (allowed-method? method)
@@ -315,7 +338,7 @@
 ;;        (la (+ addr lo))
 ;;         (len (- hi lo)))
 ;;    (pointer->bytevector (make-pointer la) len)))
-  
+
 (define-syntax bv-slice
   (syntax-rules (:)
     ((_ bv lo : hi)
@@ -328,14 +351,19 @@
 ;; get the unified basename both POSIX and WINDOWS
 (define (uni-basename filename)
   (substring filename
-   (1+ 
-    (string-index-right filename 
-                        (lambda (c) (or (char=? c #\\) (char=? c #\/)))))))
+             (1+ 
+              (string-index-right filename 
+                                  (lambda (c) (or (char=? c #\\) (char=? c #\/)))))))
 
 ;; FIXME: checkout-the-path only support POSIX file path
 ;; FIXME: what's the proper default mode for the dir?
 (define* (checkout-the-path path #:optional (mode #o775))
-  (let ((paths (string-split path #\/)))
+  (define (->path p)
+    (let ((pp (irregex-split "/" p)))
+      (if (char=? (string-ref p 0) #\/)
+          (cons (string-append "/" (car pp)) (cdr pp))
+          pp)))
+  (let ((paths (->path path)))
     (let lp((next paths) (last ""))
       (cond
        ((null? next) #t)
@@ -399,10 +427,10 @@
            (optimize (cdr rev-items)
                      (cons (car rev-items) tail)))
           (else (receive (strings rest) (span string? rev-items)
-                  (let ((s (string-concatenate-reverse strings)))
-                    (if (string-null? s)
-                        (optimize rest tail)
-                        (optimize rest (cons s tail))))))))
+                         (let ((s (string-concatenate-reverse strings)))
+                           (if (string-null? s)
+                               (optimize rest tail)
+                               (optimize rest (cons s tail))))))))
   (define (match->item m)
     (or (and (irregex-match-substring m 'dollar) "$")
         (let* ((name (irregex-match-substring m 'var))
@@ -445,17 +473,15 @@
 (define (bytevector-null? bv)
   ((@ (rnrs bytevectors) bytevector=?) bv #u8()))
 
-(define (prepare-headers body headers)
-  (define *default-header*
-    (if (assq-ref headers 'content-type) '() '((content-type . (text/html)))))
-  ;; FIXME: the latest Guile fixed content-length:0 bug, but 2.0.9 is not,
-  ;;        so remove it when next release.
-  (when (not body)
-    (error prepare-headers "Fatal: Something got wrong, the body shouldn't be #f!" body))
-  (let* ((check (cond ((bytevector? body) bytevector-null?)
-                      ((string? body) string-null?)))
-         (len (if (check body) '((content-length . 0)) '())))
-    `(,@*default-header* ,@headers ,@len)))
+(define (generate-modify-time t)
+  (get-local-date (cons (time-second t) (time-nanosecond t))))
+
+(define (prepare-headers headers)
+  (define *default-headers*
+    `((content-type . (text/html (charset . ,(get-conf '(server charset)))))
+      (date . ,(get-global-date))))
+  (lset-union (lambda (x y) (eq? (car x) (car y)))
+              (assq-remove! headers 'last-modified) *default-headers*))
 
 (define new-stack make-q)
 (define new-queue make-q)
@@ -471,6 +497,8 @@
 (define stack-top q-front)
 (define stack-remove! %q-remove-with-key!)
 (define stack-empty? q-empty?) 
+(define stack-length q-length)
+(define (stack->list stk) (list-copy (stack-slots stk)))
 
 (define queue-out! q-pop!)
 (define queue-in! enq!)
@@ -478,6 +506,8 @@
 (define queue-tail q-rear)
 (define queue-remove! %q-remove-with-key!)
 (define queue-empty? q-empty?)
+(define queue-length q-length)
+(define (queue->list q) (list-copy (queue-slots q)))
 
 (define* (list->stack lst #:optional (stk (new-stack))) ; NOTE: make-stack exists in Guile
   (for-each (lambda (x) (stack-push! stk x)) lst)
@@ -574,6 +604,9 @@
 (define (run-before-response! proc)
   (add-hook! *before-response-hook* proc))
 
+(define (run-before-run! proc)
+  (add-hook! *before-run-hook* proc))
+
 ;; NOTE: For `pipeline' methodology, please read my post:
 ;; http://nalaginrut.com/archives/2014/04/25/oba-pipeline-style%21
 (define (make-pipeline . procs)
@@ -624,7 +657,334 @@
       (else (throw 'artanis-err 500 "generate-kv-from-post-qstr: Fatal! Can't be here!" lst))))
   (define (-> x)
     (string-trim-both x (lambda (c) (member c '(#\sp #\: #\return)))))
-  (format #t "GGGGGGG: ~a~%" (utf8->string body))
   (map (lambda (x)
          (%convert (map -> (string-split (cook x) #\=))))
        (string-split (utf8->string body) #\&)))
+
+;; NOTE: We accept warnings, which means if warnings occurred, it'll be 200(OK) anyway, but
+;;       Artanis will throw warnings in the server-side log.
+;; NOTE: We *DO NOT* accept errors, which means if errors occurred, Artanis will throw 500.
+(define (handle-proper-owner file uid gid)
+  (define-syntax-rule (print-the-warning exe reason)
+    (format (current-error-port) "[WARNING] '~a' encountered system error: ~s~%" exe reason))
+  (define-syntax-rule (->err-reason exe reason)
+    (format #f "'~a' encoutered system error: ~s" exe reason))
+  (catch 'system-error
+         (lambda ()
+           (chown file (or uid (getuid)) (or gid (getgid))))
+         (lambda (k . e)
+           (let ((exe (car e))
+                 (reason (caaddr e)))
+             (match (cons k reason)
+               ('(system-error . "Operation not permitted")
+                (print-the-warning exe reason)
+                (display
+                 "Maybe you run Artanis as unprivileged user? (say, not as root)\n"
+                 (current-error-port)))
+               ('(system-error . "No such file or directory")
+                (throw 'artanis-err 500 (->err-reason exe reason) file))
+               (else (apply throw k e)))))))
+
+;; According to wiki, here's the standard format of data_url_scheme:
+;; data:[<MIME-type>][;charset=<encoding>][;base64],<data>
+(define* (generate-data-url bv/str #:key (mime 'application/octet-stream)
+                            (crypto 'base64) (charset 'utf-8))
+  (define-syntax-rule (->crypto)
+    (match crypto
+      ((or 'base64 "base64") ";base64")
+      (else "")))
+  (define-syntax-rule (->charset)
+    (if (or (string? charset) (symbol? charset))
+        (format #f ",charset=~a" charset)
+        ""))
+  (define-syntax-rule (->mime)
+    (match mime
+      (`(guess ,ext) (or (mime-guess ext) 'application/octet-stream))
+      ((or (? symbol?) (? string?))
+       (or (and (get-conf 'debug-mode) (mime-check mime) (format #f "~a" mime))
+           (format #f "~a" mime)))
+      (else (throw 'artanis-err 500
+                   "generate-data-url: Invalid MIME! Should be symbol or string"
+                   mime))))
+  (let ((b64 (base64-encode bv/str)))
+    (string-concatenate (list "data:" (->mime) (->crypto) (->charset) "," b64))))
+
+(define (verify-ENTRY entry)
+  (cond
+   ((not (file-exists? entry)) #f)
+   (else
+    (let* ((line (call-with-input-file entry read-line))
+           (m (string-match "Artanis top-level: (.*)" line)))
+      (and m (string=? (match:substring m 1) (dirname entry)))))))
+
+(define-syntax draw-expander
+  (syntax-rules (rule options method)
+    ((_ (options options* ...) rest rest* ...)
+     `(,@(list options* ...) ,@(draw-expander rest rest* ...)))
+    ((_ (method method*) rest rest* ...)
+     `((method ,'method*) ,@(draw-expander rest rest* ...)))
+    ((_ (rule url) rest rest* ...)
+     `((rule ,url) ,@(draw-expander rest rest* ...)))
+    ((_ handler) (list handler))))
+
+(define (remove-ext str)
+  (let ((i (string-contains str ".")))
+    (substring str 0 i)))
+
+(define (scan-app-components component)
+  (let ((toplevel (current-toplevel)))
+    (map (lambda (f) (string->symbol (remove-ext f)))
+         (scandir (format #f "~a/app/~a/" toplevel component)
+                  (lambda (f)
+                    (not (or (string=? f ".")
+                             (string=? f ".."))))))))
+
+(define (cache-this-route! url meta)
+  (define (write-header port)
+    (format port ";; Do not touch anything!!!~%")
+    (format port ";; All things here should be automatically handled properly!!!~%"))
+  (define route-cache (string-append (current-toplevel) "/tmp/cache/route.cache"))
+  (when (or (not (file-exists? route-cache))
+            (and (not url) (not meta))) ; for regenerating route cache
+    (format (artanis-current-output) "Route cache is missing, regenerating...~%")
+    (call-with-output-file route-cache
+      (lambda (port) (write-header port) (write '() port))))
+  (when (and url meta)
+    (let ((rl (call-with-input-file route-cache read)))
+      (delete-file route-cache)
+      (call-with-output-file route-cache
+        (lambda (port)
+          (flock port LOCK_EX)
+          (write-header port)
+          (if (eof-object? rl)
+              (write '() port)
+              (write (assoc-set! rl url (drop-right meta 1)) port))
+          (flock port LOCK_UN))))))
+
+(define (dump-route-from-cache)
+  (define toplevel (current-toplevel))
+  (define route-cache (string-append toplevel "/tmp/cache/route.cache"))
+  (define route (string-append toplevel "/.route"))
+  (define (load-customized-router)
+    (let ((croute (string-append toplevel "conf/route")))
+      (cond
+       ((not (file-exists? croute)) #t) ; No customized route
+       (else
+        (use-modules (artanis mvc route)) ; black magic to make Guile happy
+        (load croute)))))
+  (when (file-exists? route) (delete-file route))
+  (when (not (file-exists? route-cache))
+        (cache-this-route! #f #f)
+        (dump-route-from-cache))
+  (let ((rl (call-with-input-file route-cache read)))
+    (cond
+     ((eof-object? rl)
+      (cache-this-route! #f #f)
+      (dump-route-from-cache))
+     (else
+      (call-with-output-file route
+        (lambda (port)
+          (for-each (lambda (r)
+                      (let* ((meta (cdr r))
+                             (rule (assq-ref meta 'rule))
+                             (method (assq-ref meta 'method)))
+                        (format port "~2t(~a ~s)~%"
+                                (if method method 'get)
+                                (if rule rule (car r)))))
+                    rl)))
+      ;; load customized router
+      (load-customized-router)))))
+
+(define* (delete-directory dir #:optional (checkonly? #f))
+  (cond
+   ((and (file-is-directory? dir) (file-exists? dir))
+    (system (format #f "rm -f ~a" dir)))
+   (else
+    (and (not checkonly?)
+         (error delete-directory "Not a directory or doesn't exist " dir)))))
+
+;; TODO: handle it more elegantly
+(define* (handle-existing-file path #:optional (dir? #f))
+  (let* ((pp (if dir? (dirname path) path))
+         (component (basename (dirname pp)))
+         (name (car (string-split (basename pp) #\.))))
+    (cond
+     ((draw:is-force?)
+      (if (file-is-directory? path)
+          (delete-directory path)
+          (delete-file path)))
+     ((draw:is-skip?)
+      (format (artanis-current-output) "skip ~10t app/~a/~a~%" component name))
+   (else
+    (format (artanis-current-output)
+            "~a `~a' exists! (Use --force/-f to overwrite or --skip/-s to ignore)~%"
+            (string-capitalize component) name)
+    (exit 1)))))
+
+;; Check if all methods are valid
+(define (check-drawing-method lst)
+  (define errstr "Invalid drawing method, shouldn't contain '/' ")
+  (for-each (lambda (name)
+              (when (not (irregex-match "[^/]+" name))
+                    (error check-drawing-method errstr name)))
+            lst)
+  lst)
+
+(define (subbv->string bv encoding start end)
+  (call-with-output-string
+   (lambda (port)
+     (set-port-encoding! port encoding)
+     (put-bytevector port bv start (- end start)))))
+
+(define* (bv-u8-index bv u8 #:optional (time 1))
+  (let ((len (bytevector-length bv)))
+    (let lp((i 0) (t 1))
+      (cond
+       ((>= i len) #f)
+       ((= (bytevector-u8-ref bv i) u8)
+        (if (= t time) i (lp (1+ i) (1+ t))))
+       (else (lp (1+ i) t))))))
+
+(define* (bv-u8-index-right bv u8 #:optional (time 1))
+  (let ((len (bytevector-length bv)))
+    (let lp((i (1- len)) (t 1))
+    (cond
+     ((< i 0) #f)
+     ((= (bytevector-u8-ref bv i) u8)
+      (if (= t time) i (lp (1- i) (1+ t))))
+     (else (lp (1- i) t))))))
+
+(define* (subbv=? bv bv2 #:optional (start 0) (end (1- (bytevector-length bv))))
+  (and (<= (bytevector-length bv2) (bytevector-length bv))
+       (let lp((i end) (j (1- (bytevector-length bv2))))
+         (cond
+          ((< i start) #t)
+          ((= (bytevector-u8-ref bv i) (bytevector-u8-ref bv2 j))
+           (lp (1- i) (1- j)))
+          (else #f)))))
+
+;; return position after delim
+(define* (bv-read-delimited bv delim #:optional (start 0) (end (bytevector-length bv)))
+  (define len (- end start -1))
+  (let lp((i start))
+    (cond
+     ((> i end) #f)
+     ((= (bytevector-u8-ref bv i) delim) i)
+     (else (lp (1+ i))))))
+
+;; return position after newline
+(define* (bv-read-line bv #:optional (start 0) (end (bytevector-length bv)))
+  (bv-read-delimited bv 10 start end))
+
+(define (put-bv port bv from to)
+  (put-bytevector port bv from (- to from 2)))
+
+;; TODO: build a char occurence indexing table
+(define (build-bv-lookup-table bv)
+  (let ((ht (make-hash-table)))
+    (for-each (lambda (i)
+                (hash-set! ht (bytevector-u8-ref bv i) #t))
+              (iota (bytevector-length bv)))
+    ht))
+
+(define Gbytes (ash 1 30))
+(define Mbytes (ash 1 20))
+(define Kbytes (ash 1 10))
+(define (filesize size) 
+  (cond
+   ((>= size Gbytes)
+    (format #f "~,1fGiB" (/ size Gbytes)))
+   ((>= size Mbytes)
+    (format #f "~,1fMiB" (/ size Mbytes)))
+   ((>= size Kbytes)
+    (format #f "~,1fKiB" (/ size Kbytes)))
+   (else (format #f "~a Bytes" size))))
+
+(define* (plist-remove lst k #:optional (no-value? #f))
+  (let lp((next lst) (kk '__) (ret '()))
+    (cond
+     ((null? next) (values (reverse ret) kk))
+     ((eq? (car next) k)
+      (if no-value?
+          (lp (cdr next) (car next) ret)
+          (lp (cddr next) (list (car next) (cadr next)) ret)))
+     (else (lp (cdr next) kk (cons (car next) ret))))))
+
+(define *name-re* (string->sre "([^.]+)\\.scm"))
+(define (gen-migrate-module-name f)
+  (cond
+   ((irregex-search *name-re* (basename f))
+    => (lambda (m) (irregex-match-substring m 1)))
+   (else (throw 'artanis-err 500
+                "Migrate: wrong parsing of module name, shouldn't be here!" f))))
+
+(define (try-to-load-migrate-cache name)
+  (let ((file (format #f "~a/tmp/cache/migration/~a.scm" (current-toplevel) name)))
+    (cond
+     ((file-exists? file) (load file))
+     (else
+      (format (artanis-current-output)
+              "[WARN] No cache for migration of `~a'~%" name)
+      (format (artanis-current-output)
+              "Run `art migrate up ~a', then try again!~%" name)))))
+
+(define (flush-to-migration-cache name fl)
+  (let ((file (format #f "~a/tmp/cache/migration/~a.scm" (current-toplevel) name)))
+    (when (file-exists? file) (delete-file file))
+    (call-with-output-file file
+      (lambda (port)
+        (format port "(define-~a~%" name)
+        (for-each
+         (lambda (ft) (format port "~2t~a~%" ft))
+         fl)
+        (format port "~2t)~%")))))
+
+(define (gen-local-conf-file)
+  (format #f "~a/conf/artanis.conf" (current-toplevel)))
+
+(define-syntax-rule (with-dbd dbd0 body ...)
+  (let ((dbd1 (get-conf '(db dbd))))
+    (cond
+     ((or (and (list? dbd0) (memq dbd1 dbd0)) (eq? dbd1 dbd0)) body ...)
+     (else
+      (throw 'artanis-err 500
+             (format #f "This is only supported by `~a', but the current dbd is `~a'"
+                     dbd0 dbd1)
+             'body ...)))))
+
+(define %libc-errno-pointer
+  ;; Glibc's 'errno' pointer.
+  (let ((errno-loc (dynamic-func "__errno_location" (dynamic-link))))
+    (and errno-loc
+         (let ((proc (pointer->procedure '* errno-loc '())))
+           (proc)))))
+
+(define errno
+  (if %libc-errno-pointer
+      (let ((bv (pointer->bytevector %libc-errno-pointer (sizeof int))))
+        (lambda ()
+          "Return the current errno."
+          ;; XXX: We assume that nothing changes 'errno' while we're doing all this.
+          ;; In particular, that means that no async must be running here.
+
+          ;; Use one of the fixed-size native-ref procedures because they are
+          ;; optimized down to a single VM instruction, which reduces the risk
+          ;; that we fiddle with 'errno' (needed on Guile 2.0.5, libc 2.11.)
+          (let-syntax ((ref (lambda (s)
+                              (syntax-case s ()
+                                ((_ bv)
+                                 (case (sizeof int)
+                                   ((4)
+                                    #'(bytevector-s32-native-ref bv 0))
+                                   ((8)
+                                    #'(bytevector-s64-native-ref bv 0))
+                                   (else
+                                    (error "unsupported 'int' size"
+                                           (sizeof int)))))))))
+            (ref bv))))
+      (lambda () 0)))
+
+(define (c/struct-sizeof meta)
+  (apply +
+         (map (lambda (m) (if (list? m) (c/struct-sizeof m) (sizeof m)))
+              meta)))
